/*
 * which.c
 * Copyright (C) 1996-2001 Particle Corporation.
 * Alex S. (Particle)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * - September 16th, 2001 - Alex S. (Particle)
 */

/**
 * this is a really simple Win32 version of the
 * UNIX which utility; it looks for a file in the
 * path; and reports it when it is found.
 *
 * used mostly to figure out which program is being
 * executed... for example, "which java" surprisingly
 * finds the java.exe in c:\windows\system32\java.exe
 * and not the one in d:\jdk1.3\bin\java.exe
 *
 * example:
 * D:\temp>which edit
 * c:\windows\system32\edit.com
 * D:\temp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/**
 * checks to see if some file is there...
 * returns non-zero if a readable file is found.
 */
int checkFile1(char* s){
    FILE* f = fopen(s,"rb");
    if(f){
        fclose(f);
        return 1;
    }
    return 0;
}

// file extensions to look for (if need more, just add to this array)
// (note: it must end with a 0 to indicate the end...)
char* PATHEXT[] = {"",".com",".exe",".bat",".cmd",".vbs",".vbe",".js",
    ".jse",".wsf",".wsh",".pl",0};

/**
 * checks to see if some file is there, using all possible
 * extensions from the list above. Returns non-zero is
 * some file is found. Also prints out exactly the file
 * found (with proper extension).
 */
int checkFile(char* s){
    char path[8192];    // this should be enough
    int i;
    for(i=0;PATHEXT[i];i++){
        strcpy(path,s);
        strcat(path,PATHEXT[i]);
        if(checkFile1(path)){
            printf("%s\n",path);
            return 1;
        }
    }
    return 0;
}

/**
 * compare all strings in lower case (Windows file
 * system is case insensitive.) This function lower
 * cases a string.
 */
void strtolower(char* s){
    while(*s){
        *s = tolower(*s);
        s++;
    }
}

/**
 * main to do all the major stuff of this simple program.
 */
int main(int argc,char** argv,char** env){

    int i;
    char path[8192];    // this should be enough
    char* pathorig;
    char* pathcopy;

    // check args (no support for more advanced options)
    if(argc < 2){
        printf("%s [progname]\n",argv[0]);
        return -1;
    }

    // check easy case... (file is in current dir)
    // [note that while UNIX doesn't pick up things from
    // current directory for execution if it's not in the
    // PATH, windows will gladly do it. This we need this.]
    if(checkFile(argv[1]))
        return 0;

    // harder case; file is in path...

    //find path variable in environment
    pathorig = 0;
    for(i=0;env[i];i++){
        strtolower(env[i]);
        // (kind of stupid code; but it works)
        if(!strncmp("path=",env[i],5)){
            pathorig = env[i];
            break;
        }
    }

    // if there is no PATH
    // (this is consistant behavior to UNIX which, when PATH is not defined)
    if(!pathorig){
        printf("which: no %s in ((null))\n",argv[1]);
        return -1;
    }

    pathorig+=5;    // get rid of "path=" prefix
    pathcopy = pathorig;    // save a copy for possible later output

    // copy 1 char at a time (really stupid parsing)
    for(i=0;path[i] = *(pathorig++);i++){
        if(path[i] == ';'){   // if hit path seperator; check for file in that dir
            path[i] = '\0';
            if(i != 0 && path[i-1] == '\\')
                path[i-1] = '\0';
            strcat(path,"\\");
            strcat(path,argv[1]);
            if(checkFile(path)) // check for file in this dir (all possible extensions)
                return 0;
            i=-1; // for loop will be incremented to make it a nice 0.
        }
    }

    // gone through all path; now finish it up.
    // (last directory in path probably didn't have a path seperator after it)
    if(i >= 0){
        path[i] = '\0';
        if(i != 0 && path[i-1] == '\\')
            path[i-1] = '\0';
        strcat(path,"\\");
        strcat(path,argv[1]);
        if(checkFile(path))
            return 0;
    }

    // not found. better luck next time ;-)
    printf("which: no %s in (%s)\n",argv[1],pathcopy);
    return 0;
}

